unit RosiCompGlobalCfg;

{<
  Global configuration - define global components, variables and settings
  for all RosiComponent package and shared forms.

  Updated version compatible with Delphi2010+.
  @author(Rosinsky Software - <http://www.rosinsky.cz/delphi>)
  @created(Jan 2013)
  @lastmod(Oct 2016)
}

interface

{$INCLUDE _rosi.inc}
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Themes, DB, Grids, DBGrids, ShlObj, Math;

const
  // global constant
  cStrEOL = #13#10;
  rDefBoolValue = '1;0';
  rRunHTTP = 'http://';
  rRunMailTo = 'mailto:';
  rActualDateAbr = '$Date';
  rHTMLHintImageTag = '<<HINTIMG>>';

  // configuration constant
  RosiCompConfig_Auto = 0;
  RosiCompConfig_On = 1;
  RosiCompConfig_Off = -1;

type
  { define class for storing information of one anchor, used by rHTMLLabel and rHTMLDrawText }
  TAnchorInfo = class
    LinkType: string;
    LinkName: string;
    Rect: TRect;
  end;

  { define event type for click on anchor, used by rHTMLLabel and rHTMLDrawText }
  TAnchorClickEvent = procedure(Sender: TObject; LinkName: string) of object;
  { define event type for getting of parameter value, used by rHTMLLabel and rHTMLDrawText }
  TGetParameter = procedure(Sender: TObject; var ParamStr: string) of object;

  { define frame types, used by rGroupBox and rHTMLListView }
  TFrameType = (ft3D, ftSingleLine, ftDoubleLine, ftNone);
  { define gradient types, used by rGroupBox }
  TGradientType = (gtNone, gtVertical, gtHorizontal);
  { define event type for help invoke, used by shared forms }
  TProcOnHelp = procedure(Form: TForm; HelpContext: integer) of object;
  { define event type after click on link, used by shared rDBGrid and rDBRecView }
  TLinkClickEvent = procedure(Sender: TObject; DataSet: TDataSet; Column: TColumn; Field: TField; Value: Variant) of object;
  { define action type after creating of report, used by RTFReport2, rXMLReport, TXTExport, SLKExport, rXLSExport }
  TActionAfter = (aaNone, aaShow, aaPrint);

  { list of possible action for DateTimePicker events }
  TDateTimePickerAction = (dpaSetDate, dpaGetDate, dpaShow, dpaHide);
  { event type for creating of DateTimePicker }
  TDateTimePickerCreate = procedure(Sender: TObject; var DateTimePicker: TWinControl; OnChange: TNotifyEvent; OnClose: TNotifyEvent) of object;
  { event type for DateTimePicker action }
  TDateTimePickerEvent = procedure(Sender: TObject; DateTimePicker: TWinControl; Action: TDateTimePickerAction; var DateTime: TDateTime; var Done: boolean)
    of object;

  { event type for DBControls used in DoExit event }
  TExitControlEvent = procedure (Sender: TObject; Field: TField; var CanExit: boolean) of object;

  { Report exception class }
  TReportException = class(Exception)
  end;

  { define list of defined grid styles for rDBGrid (can be expanded by developer) }
  TGridLookStyles = (grs_Default, grsBlueOffice, grsBlueNavy, grsBlueVista, grsSilverLight, grsSilverDark, grsSilverMonochrome, grsBlackOffice, grsOlive,
    grsYellow, grsOrange, grsGreen, grsBrown, grsKhaki);

  { list of possible LookStyles for rDBGrid }
  TGlyphLookStyles = (glsStandard, glsFlatBlack, glsFlatGray);

const
  // default values
  rGroupBoxDefFrame = ft3D;
  rGroupBoxDefSeparator = ';';
  rPageExSpace = 2;

type
  { non visual components witch define custom glyphs for Rosi components }
  TrCustomGlyphs = class(TComponent)
  private
    FSortUp: TBitmap;
    FSortDown: TBitmap;
    FFilter: TBitmap;
    FFilterActive: TBitmap;
    FCheckFalse: TBitmap;
    FCheckTrue: TBitmap;
    FMemoEmpty: TBitmap;
    FMemo: TBitmap;
    FPictureEmpty: TBitmap;
    FPicture: TBitmap;
    FBtnFormat: TBitmap;
    FBtnEditWin: TBitmap;
    FBtnLookupSelect: TBitmap;
    FBtnLookupDetail: TBitmap;
    FBtnEllipsis: TBitmap;
    FUseAsGlobalGlyphs: boolean;
    procedure SetSortUp(Value: TBitmap);
    procedure SetSortDown(Value: TBitmap);
    procedure SetFilter(Value: TBitmap);
    procedure SetFilterActive(Value: TBitmap);
    procedure SetCheckFalse(Value: TBitmap);
    procedure SetCheckTrue(Value: TBitmap);
    procedure SetMemoEmpty(Value: TBitmap);
    procedure SetMemo(Value: TBitmap);
    procedure SetPictureEmpty(Value: TBitmap);
    procedure SetPicture(Value: TBitmap);
    procedure SetBtnFormat(Value: TBitmap);
    procedure SetBtnEditWin(Value: TBitmap);
    procedure SetBtnLookupSelect(Value: TBitmap);
    procedure SetBtnLookupDetail(Value: TBitmap);
    procedure SetBtnEllipsis(Value: TBitmap);
    procedure SetUseAsGlobalGlyphs(const Value: boolean);
  public
    constructor Create(AOwner: TComponent); override; {< @exclude }
    destructor Destroy; override; {< @exclude }
  published
    property SortUp: TBitmap read FSortUp write SetSortUp;
    property SortDown: TBitmap read FSortDown write SetSortDown;
    property Filter: TBitmap read FFilter write SetFilter;
    property FilterActive: TBitmap read FFilterActive write SetFilterActive;
    property CheckFalse: TBitmap read FCheckFalse write SetCheckFalse;
    property CheckTrue: TBitmap read FCheckTrue write SetCheckTrue;
    property MemoEmpty: TBitmap read FMemoEmpty write SetMemoEmpty;
    property Memo: TBitmap read FMemo write SetMemo;
    property PictureEmpty: TBitmap read FPictureEmpty write SetPictureEmpty;
    property Picture: TBitmap read FPicture write SetPicture;
    property BtnFormat: TBitmap read FBtnFormat write SetBtnFormat;
    property BtnEditWin: TBitmap read FBtnEditWin write SetBtnEditWin;
    property BtnLookupSelect: TBitmap read FBtnLookupSelect write SetBtnLookupSelect;
    property BtnLookupDetail: TBitmap read FBtnLookupDetail write SetBtnLookupDetail;
    property BtnEllipsis: TBitmap read FBtnEllipsis write SetBtnEllipsis;
    { set the TrCustomGlyphs as global setting for all Rosi components }
    property UseAsGlobalGlyphs: boolean read FUseAsGlobalGlyphs write SetUseAsGlobalGlyphs default false;
  end;

  { global record which contain and return needed glyph from Global or Custom glyph component }
  TrGlyphsRec = record
    BmpArray: array of TBitmap;
    function BmpFromIndex(GlyphIndex: integer; CustomGlyphs: TrCustomGlyphs = nil): TBitmap;
    function SortIndex(Desc: boolean): integer;
    function SortWidth: integer;
    function CheckIndex(Checked: boolean): integer;
    function MemoIndex(Empty: boolean; TypeIndex: integer): integer;
    function PictureIndex(Empty: boolean; TypeIndex: integer): integer;
    function FilterIndex(Active: boolean): integer;
    function TitleExIndex(FilterActive: boolean): integer;
    function EllipsisIndex: integer;
    function FilterWidth: integer;
    function RecordIndicatorIndex(ActiveRecord, Multiselected, ReadOnly: boolean; State: TDataSetState): integer;
    function ButtonIndex(ButtonName: string): integer;
  end;

  TrDBToolCfg = record
    SQLScript_StringFieldQuotes: string;
    SQLScript_CommandSeparator: string;
    SQLScript_BooleanTrueValue: string;
    SQLScript_BooleanFalseValue: string;
    FileDialog_Options: TOpenOptions;
    FileDialog_InitDialogEvent: TNotifyEvent;
    rDlg_DefaultFont: string;
    rDlg_DefaultSize: integer;
    rDlg_AnchorClick: TAnchorClickEvent;
    rDlg_GridDrawingStyle: TGridDrawingStyle;
    rDlg_BlockStoringGridFields: boolean;
    rpictool_LastFileName: string;
    BlobField_ftBlobAsMemo: boolean;
  end;

  TPropSaveCfg = record
    Suffix_BlockSaveItems: string;
    FieldList_UseNewWidth: string;
    Grids_UseGlobalSetting: boolean;
  end;

  TrGroupBoxCfg = record
    DefaultCaptionColor: TColor;
    DefaultFontColor: TColor;
    DefaultCaptionHeight: integer;
    DefaultBkColor: TColor;
    UseSimpleTabColor: boolean;
  end;

  TrDBComponentsCfg = record
    JPGAsDefault: boolean;
    VisibilityFromField_AutoHide: boolean;
    VisibilityFromField_AutoShow: boolean;
    ExitControlEvent: TExitControlEvent;
    function DoExitControl(Sender: TControl; F: TField): boolean;
  end;

  TrDBGridCfg = record
    DefaultDrawingStyle: TGridDrawingStyle;
    DefaultGridLookStyle: TGridLookStyles;
    DefaultGlyphLookStyle: TGlyphLookStyles;
    FilterListUseColumnAlignment: boolean;
    FilterListBoldSelectAll: boolean;
    CellSingleLineEOLSeparator: string;
  end;

  TrDBFieldsCfg = record
    OnSetDefaultGridColumns: TNotifyEvent;
    SetFixedCol: boolean;
  end;

  TrDBFilterCfg = record
    LikeChar: Char;
    CompoundLikeChar: Char;
    NullEqualStr: string;
    DateFormat: string;
    BooleanString: array [boolean] of string;
    TableQuoteStart: Char;
    TableQuoteStop: Char;
    FieldQuoteStart: Char;
    FieldQuoteStop: Char;
    ShowDelQuotes: boolean;
    ShowCaseSensitive: boolean;
  end;

  TrDBFindCfg = record
    ShowDelQuotes: boolean;
    OnFoundEvent: TNotifyEvent;
    LastOptIndex: integer;
    LastCaseSens: boolean;
    LastFindStr: string;
  end;

  TrDBGridPropsCfg = record
    HideCellParams: boolean;
    HideTitleFilterBtn: boolean;
    HideGridStyles: boolean;
  end;

  TrDBExportCfg = record
    UseDisableControl: boolean;
  end;

  TrDBPrintCfg = record
    PrintData_Dir: string;
  end;

  TrDBPrintDirectCfg = record
    HMarginMM: integer;
    VMarginMM: integer;
    LineSpaceRatio: double;
    PreviewMaxWidth: integer;
  end;

  TrDBSortCfg = record
    SortFieldList: boolean;
    DefaultPath: string;
  end;

  TRTFRepDlgCfg = record
    LastLangVerIndex: integer;
    ExportExcelNoFileName: boolean;
  end;

  TLoadFieldsPropsProc = procedure(DS: TDataSet) of object;

  TrDBFieldDefaultsCfg = record
    DefaultFloatDisplayFormat: string;
    DefaultFloatEditFormat: string;
    DefaultBoolDisplayFormat: string;
    DefaultDateTimeDisplayFormat: string;
    DefaultTimeDisplayFormat: string;
    LoadFieldsPropsProc: TLoadFieldsPropsProc;
    InternalSQLLog: TStrings;
    InternalLogSeparator: string;
  end;

  { define global configuration record }
  TRosiCompConfig = record
    Global_CustomGlyphs: TrCustomGlyphs;
    Global_PropSaveFile: TObject;
    Global_ProcOnHelp: TProcOnHelp;
    Global_HelpDirectory: string;
    Global_HTMLHintComponent: TObject;
    Global_HTMLHintPicture: TPicture;

    PropSaveCfg: TPropSaveCfg;
    rGroupBoxCfg: TrGroupBoxCfg;
    rDBComponentsCfg: TrDBComponentsCfg;
    rDBGridCfg: TrDBGridCfg;
    rDBToolCfg: TrDBToolCfg;
    rDBFieldDefaultsCfg: TrDBFieldDefaultsCfg;

    rDBFieldsCfg: TrDBFieldsCfg;
    rDBFilterCfg: TrDBFilterCfg;
    rDBFindCfg: TrDBFindCfg;
    rDBGridPropsCfg: TrDBGridPropsCfg;
    rDBExportCfg: TrDBExportCfg;
    rDBPrintCfg: TrDBPrintCfg;
    rDBPrintDirectCfg: TrDBPrintDirectCfg;
    rDBSortCfg: TrDBSortCfg;
    RTFRepDlgCfg: TRTFRepDlgCfg;
  end;

  { define record for one GridLook style }
  TGridLookStyleDef = record
    StyleName: string;
    GradientStartColor: TColor;
    GradientEndColor: TColor;
    TitleFontColor: TColor;
    TextFontColor: TColor;
    TextBkColor: TColor;
    TextBkColorActiveRow: TColor;
    TextBkColorOddRow: TColor;
  end;


var
  { global variable for global configuration, developer can set own values }
  RosiCompConfig: TRosiCompConfig;
  { global array variable containing grid style values }
  GridLookStyleDefs: array [0 .. 13] of TGridLookStyleDef;
  { global variable containing standard glyphs for all Rosi components }
  rGlyphs: TrGlyphsRec;

  { return filename used for storing of design time dialog settings }
function GetRosiCompIniFile: string;
{ loads list of possible GridLookStyles }
procedure LoadGridLookStyleList(L: TStrings);

procedure Register; {< @exclude }

implementation

{$R *.res}
uses rtool, rpictool, AppDirCfg;

type
  { @exclude }
  THackWinControl = class(TWinControl)
  end;

procedure Register;
begin
  RegisterComponents('Rosi', [TrCustomGlyphs]);
end;

const
  // resource names stored in RosiCompGlobalCfg.res
  rMaxBmpArr = 33;
  rResName: array [0 .. rMaxBmpArr] of string = ('RDBMARKUP', 'RDBMARKDOWN', 'RDBCHECKEMPTY', 'RDBCHECKED', 'RDBMEMOEMPTY1', 'RDBMEMOEMPTY2', 'RDBMEMOEMPTY3',
    'RDBMEMO1', 'RDBMEMO2', 'RDBMEMO3', 'RDBPICTUREEMPTY1', 'RDBPICTUREEMPTY2', 'RDBPICTUREEMPTY3', 'RDBPICTURE1', 'RDBPICTURE2', 'RDBPICTURE3',
    'RDBTITLEEXSUM', 'RDBTITLEEXFILTERED', 'RDBINDICBROWSE', 'RDBINDICEDIT', 'RDBINDICINSERT', 'RDBINDICREADONLY', 'RDBINDICSELECTED', 'RDBINDICSELACTIVE',
    'RDBFILTERBTN', 'RDBFILTERBTNA', 'RBTNCBADD12', 'RBTNCBINS12', 'RBTNFORMAT10', 'RBTNEDITWIN10', 'RBTNSEEK16', 'RBTNERASE16', 'RDBLOOKUPDETAIL12',
    'RDBLOOKUPSELECT12');

  // resource indexes in rResName
  rGlyphIdxSort = 0;
  rGlyphIdxCheck = 2;
  rGlyphIdxMemo = 4;
  rGlyphIdxPicture = 10;
  rGlyphIdxTitleEx = 16;
  rGlyphIdxIndic = 18;
  rGlyphIdxIndicSel = 22;
  rGlyphIdxFilter = 24;
  rGlyphIdxButtonsMin = 26;
  rGlyphIdxButtonsMax = rMaxBmpArr;
  rGlyphIdxEllipsis = 101;

  { loads all glyphs from resources to rGlyphs }
procedure LoadStandardGlyphs;
var
  i: integer;
begin
  SetLength(rGlyphs.BmpArray, rMaxBmpArr + 1);
  for i:=0 to rMaxBmpArr do
  begin
    rGlyphs.BmpArray[i]:=TBitmap.Create;
    rGlyphs.BmpArray[i].Transparent:=not(i in [rGlyphIdxCheck, rGlyphIdxCheck + 1]);
    if rResName[i] <> '' then
      rGlyphs.BmpArray[i].LoadFromResourceName(HInstance, rResName[i]);
  end;
end;

{ free all resources from rGlyphs }
procedure FreeStandardGlyphs;
var
  i: integer;
begin
  for i:=0 to rMaxBmpArr do
    rGlyphs.BmpArray[i].Free;
end;

{ return glyph according Index from standard glyps or CustomGlyphs component }
function TrGlyphsRec.BmpFromIndex(GlyphIndex: integer; CustomGlyphs: TrCustomGlyphs = nil): TBitmap;
var
  CustomBmp: TBitmap;

  procedure SetCustomGlyphs(CG: TrCustomGlyphs);
    procedure SetBmp(B: TBitmap);
    begin
      if Assigned(B) and (B.Width > 0) then
        CustomBmp:=B;
    end;

  begin
    CustomBmp:=nil;
    if not Assigned(CG) then
      Exit;
    case GlyphIndex of
      rGlyphIdxSort:
        SetBmp(CG.SortUp);
      rGlyphIdxSort + 1:
        SetBmp(CG.SortDown);
      rGlyphIdxFilter:
        SetBmp(CG.Filter);
      rGlyphIdxFilter + 1:
        SetBmp(CG.FilterActive);
      rGlyphIdxCheck:
        SetBmp(CG.CheckFalse);
      rGlyphIdxCheck + 1:
        SetBmp(CG.CheckTrue);
      rGlyphIdxMemo .. rGlyphIdxMemo + 2:
        SetBmp(CG.MemoEmpty);
      rGlyphIdxMemo + 3 .. rGlyphIdxMemo + 5:
        SetBmp(CG.Memo);
      rGlyphIdxPicture .. rGlyphIdxPicture + 2:
        SetBmp(CG.PictureEmpty);
      rGlyphIdxPicture + 3 .. rGlyphIdxPicture + 5:
        SetBmp(CG.Picture);
      rGlyphIdxButtonsMin + 2:
        SetBmp(CG.BtnFormat);
      rGlyphIdxButtonsMin + 3:
        SetBmp(CG.BtnEditWin);
      rGlyphIdxButtonsMin + 6:
        SetBmp(CG.BtnLookupDetail);
      rGlyphIdxButtonsMin + 7:
        SetBmp(CG.BtnLookupSelect);
      rGlyphIdxEllipsis:
        SetBmp(CG.BtnEllipsis);
    end;
  end;

begin
  SetCustomGlyphs(CustomGlyphs);
  if CustomBmp = nil then
    SetCustomGlyphs(RosiCompConfig.Global_CustomGlyphs);
  if (CustomBmp = nil) and (GlyphIndex in [0 .. rMaxBmpArr]) then
    Result:=BmpArray[GlyphIndex]
  else
    Result:=CustomBmp;
end;

{ return GlyphIndex for Sort glyph }
function TrGlyphsRec.SortIndex(Desc: boolean): integer;
begin
  Result:=rGlyphIdxSort;
  if Desc then
    Inc(Result);
end;

{ return width of Sort glyph }
function TrGlyphsRec.SortWidth: integer;
begin
  Result:=Max(BmpArray[rGlyphIdxSort].Width, BmpArray[rGlyphIdxSort + 1].Width);
end;

{ return GlyphIndex for Check glyph }
function TrGlyphsRec.CheckIndex(Checked: boolean): integer;
begin
  Result:=rGlyphIdxCheck;
  if Checked then
    Inc(Result);
end;

{ return GlyphIndex for Memo glyph }
function TrGlyphsRec.MemoIndex(Empty: boolean; TypeIndex: integer): integer;
begin
  Result:=rGlyphIdxMemo + TypeIndex;
  if not Empty then
    Inc(Result, 3);
end;

{ return GlyphIndex for Picture glyph }
function TrGlyphsRec.PictureIndex(Empty: boolean; TypeIndex: integer): integer;
begin
  Result:=rGlyphIdxPicture + TypeIndex;
  if not Empty then
    Inc(Result, 3);
end;

{ return GlyphIndex for Filter glyph }
function TrGlyphsRec.FilterIndex(Active: boolean): integer;
begin
  Result:=rGlyphIdxFilter;
  if Active then
    Inc(Result);
end;

{ return width of filter glyph }
function TrGlyphsRec.FilterWidth: integer;
begin
  Result:=Max(BmpArray[rGlyphIdxFilter].Width, BmpArray[rGlyphIdxFilter + 1].Width);
end;

{ return GlyphIndex for TitleEx glyph }
function TrGlyphsRec.TitleExIndex(FilterActive: boolean): integer;
begin
  Result:=rGlyphIdxTitleEx;
  if FilterActive then
    Inc(Result);
end;

{ return GlyphIndex for Ellipsis button glyph }
function TrGlyphsRec.EllipsisIndex: integer;
begin
  Result:=rGlyphIdxEllipsis;
end;

{ return GlyphIndex for RecordIndicator glyph }
function TrGlyphsRec.RecordIndicatorIndex(ActiveRecord, Multiselected, ReadOnly: boolean; State: TDataSetState): integer;
begin
  Result:=-1;
  if ActiveRecord then
  begin
    case State of
      dsEdit:
        Result:=rGlyphIdxIndic + 1;
      dsInsert:
        Result:=rGlyphIdxIndic + 2;
      else
        begin
          if Multiselected then
            Result:=rGlyphIdxIndicSel + 1
          else
            // check readonly indic
            if ReadOnly then
              Result:=rGlyphIdxIndic + 3
            else
              Result:=rGlyphIdxIndic;
        end;
      end;
  end
  else
    if Multiselected then
      Result:=rGlyphIdxIndicSel;
end;

{ return GlyphIndex for button glyph according button name }
function TrGlyphsRec.ButtonIndex(ButtonName: string): integer;
var
  i: integer;
begin
  Result:=-1;
  ButtonName:=UpperCase(ButtonName);
  for i:=rGlyphIdxButtonsMin to rGlyphIdxButtonsMax do
    if Pos(ButtonName, rResName[i]) > 0 then
      Result:=i;
end;

// ==============================================================================
// TrCustomGlyphs functions
constructor TrCustomGlyphs.Create(AOwner: TComponent);
begin
  inherited;
  FSortUp:=TBitmap.Create;
  FSortUp.Transparent:=true;
  FSortDown:=TBitmap.Create;
  FSortDown.Transparent:=true;
  FFilter:=TBitmap.Create;
  FFilter.Transparent:=true;
  FFilterActive:=TBitmap.Create;
  FFilterActive.Transparent:=true;
  FCheckFalse:=TBitmap.Create;
  FCheckFalse.Transparent:=true;
  FCheckTrue:=TBitmap.Create;
  FCheckTrue.Transparent:=true;
  FMemoEmpty:=TBitmap.Create;
  FMemoEmpty.Transparent:=true;
  FMemo:=TBitmap.Create;
  FMemo.Transparent:=true;
  FPictureEmpty:=TBitmap.Create;
  FPictureEmpty.Transparent:=true;
  FPicture:=TBitmap.Create;
  FPicture.Transparent:=true;

  FBtnFormat:=TBitmap.Create;
  FBtnFormat.Transparent:=true;
  FBtnEditWin:=TBitmap.Create;
  FBtnEditWin.Transparent:=true;
  FBtnLookupSelect:=TBitmap.Create;
  FBtnLookupSelect.Transparent:=true;
  FBtnLookupDetail:=TBitmap.Create;
  FBtnLookupDetail.Transparent:=true;
  FBtnEllipsis:=TBitmap.Create;
  FBtnEllipsis.Transparent:=true;

  FUseAsGlobalGlyphs:=false;
end;

destructor TrCustomGlyphs.Destroy;
begin
  FSortUp.Free;
  FSortDown.Free;
  FFilter.Free;
  FFilterActive.Free;
  FCheckFalse.Free;
  FCheckTrue.Free;
  FMemoEmpty.Free;
  FMemo.Free;
  FPictureEmpty.Free;
  FPicture.Free;

  FBtnFormat.Free;
  FBtnEditWin.Free;
  FBtnLookupSelect.Free;
  FBtnLookupDetail.Free;
  FBtnEllipsis.Free;
  inherited;
end;

procedure TrCustomGlyphs.SetSortUp(Value: TBitmap);
begin
  FSortUp.Assign(Value);
  FSortUp.Transparent:=true;
end;

procedure TrCustomGlyphs.SetSortDown(Value: TBitmap);
begin
  FSortDown.Assign(Value);
  FSortDown.Transparent:=true;
end;

procedure TrCustomGlyphs.SetFilter(Value: TBitmap);
begin
  FFilter.Assign(Value);
  FFilter.Transparent:=true;
end;

procedure TrCustomGlyphs.SetFilterActive(Value: TBitmap);
begin
  FFilterActive.Assign(Value);
  FFilterActive.Transparent:=true;
end;

procedure TrCustomGlyphs.SetCheckFalse(Value: TBitmap);
begin
  FCheckFalse.Assign(Value);
  FCheckFalse.Transparent:=true;
end;

procedure TrCustomGlyphs.SetCheckTrue(Value: TBitmap);
begin
  FCheckTrue.Assign(Value);
  FCheckTrue.Transparent:=true;
end;

procedure TrCustomGlyphs.SetMemoEmpty(Value: TBitmap);
begin
  FMemoEmpty.Assign(Value);
  FMemoEmpty.Transparent:=true;
end;

procedure TrCustomGlyphs.SetMemo(Value: TBitmap);
begin
  FMemo.Assign(Value);
  FMemo.Transparent:=true;
end;

procedure TrCustomGlyphs.SetPictureEmpty(Value: TBitmap);
begin
  FPictureEmpty.Assign(Value);
  FPictureEmpty.Transparent:=true;
end;

procedure TrCustomGlyphs.SetPicture(Value: TBitmap);
begin
  FPicture.Assign(Value);
  FPicture.Transparent:=true;
end;

procedure TrCustomGlyphs.SetBtnFormat(Value: TBitmap);
begin
  FBtnFormat.Assign(Value);
  FBtnFormat.Transparent:=true;
end;

procedure TrCustomGlyphs.SetBtnEditWin(Value: TBitmap);
begin
  FBtnEditWin.Assign(Value);
  FBtnEditWin.Transparent:=true;
end;

procedure TrCustomGlyphs.SetBtnLookupSelect(Value: TBitmap);
begin
  FBtnLookupSelect.Assign(Value);
  FBtnLookupSelect.Transparent:=true;
end;

procedure TrCustomGlyphs.SetBtnLookupDetail(Value: TBitmap);
begin
  FBtnLookupDetail.Assign(Value);
  FBtnLookupDetail.Transparent:=true;
end;

procedure TrCustomGlyphs.SetBtnEllipsis(Value: TBitmap);
begin
  FBtnEllipsis.Assign(Value);
  FBtnEllipsis.Transparent:=true;
end;

procedure TrCustomGlyphs.SetUseAsGlobalGlyphs(const Value: boolean);
begin
  if FUseAsGlobalGlyphs <> Value then
  begin
    FUseAsGlobalGlyphs:=Value;
    if FUseAsGlobalGlyphs then
    begin
      RosiCompGlobalCfg.RosiCompConfig.Global_CustomGlyphs:=self;
    end;
  end;
end;

// ==============================================================================
// Global config functions
function GetRosiCompIniFile: string;
begin
  Result:=AppDirCfg.GetSystemFolder(CSIDL_APPDATA) + 'Embarcadero';
  ForceDirectories(Result);
  Result:=Result + '\rosicomp.ini';
end;

{ define standard colors for GridLookStyle }
procedure InitGridThemeColors;
var
  Theme: TGridLookStyleDef;
  idx: integer;
begin
  idx:=0;
  Theme.StyleName:='(default)';
  Theme.GradientStartColor:=clWhite;
  Theme.GradientEndColor:=$00D6D6D6;
  Theme.TitleFontColor:=clWindowText;
  Theme.TextFontColor:=clWindowText;
  Theme.TextBkColor:=clWindow; // $00F8F8F8;
  Theme.TextBkColorActiveRow:=clInfoBk;
  Theme.TextBkColorOddRow:=clNone;
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Blue Office';
  Theme.GradientStartColor:=RGB(221, 237, 253);
  Theme.GradientEndColor:=RGB(146, 192, 224);
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=RGB(232, 239, 255);
  Theme.TextBkColorActiveRow:=RGB(254, 179, 78);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Blue Nave';
  Theme.GradientStartColor:=RGB(160, 203, 229);
  Theme.GradientEndColor:=RGB(71, 141, 190);
  Theme.TextFontColor:=RGB(0, 105, 175);
  Theme.TextBkColor:=RGB(237, 243, 247);
  Theme.TextBkColorActiveRow:=RGB(190, 213, 228);
  Theme.TextBkColorOddRow:=RGB(229, 238, 244);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Blue Vista';
  Theme.GradientStartColor:=RGB(133, 175, 192);
  Theme.GradientEndColor:=RGB(41, 107, 134);
  Theme.TitleFontColor:=clWhite;
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=RGB(246, 249, 254);
  Theme.TextBkColorActiveRow:=RGB(214, 239, 252);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Silver Light';
  Theme.GradientStartColor:=RGB(243, 244, 250);
  Theme.GradientEndColor:=RGB(156, 154, 183);
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=RGB(240, 243, 248);
  Theme.TextBkColorActiveRow:=RGB(253, 225, 141);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Silver Dark';
  Theme.GradientStartColor:=RGB(220, 219, 225);
  Theme.GradientEndColor:=RGB(151, 154, 159);
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=RGB(240, 243, 248);
  Theme.TextBkColorActiveRow:=RGB(253, 225, 141);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Silver Monochrome';
  Theme.GradientStartColor:=RGB(92, 92, 92);
  Theme.GradientEndColor:=RGB(68, 68, 68);
  Theme.TitleFontColor:=clWhite;
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=RGB(247, 247, 247);
  Theme.TextBkColorActiveRow:=RGB(152, 152, 152);
  Theme.TextBkColorOddRow:=RGB(235, 235, 235);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Black Office';
  Theme.GradientStartColor:=RGB(154, 154, 154);
  Theme.GradientEndColor:=RGB(62, 64, 62);
  Theme.TitleFontColor:=clWhite;
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=RGB(240, 243, 248);
  Theme.TextBkColorActiveRow:=RGB(253, 225, 141);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Olive';
  Theme.GradientStartColor:=RGB(251, 255, 238);
  Theme.GradientEndColor:=RGB(183, 195, 145);
  Theme.TitleFontColor:=clBlack;
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=RGB(228, 230, 199);
  Theme.TextBkColorActiveRow:=RGB(247, 201, 105);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Yellow';
  Theme.GradientStartColor:=RGB(255, 231, 77);
  Theme.GradientEndColor:=RGB(255, 209, 68);
  Theme.TitleFontColor:=RGB(141, 46, 28);
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=clWhite;
  Theme.TextBkColorActiveRow:=RGB(187, 204, 102);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Orange';
  Theme.GradientStartColor:=RGB(255, 198, 148);
  Theme.GradientEndColor:=RGB(240, 116, 18);
  Theme.TitleFontColor:=clBlack;
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=clWhite;
  Theme.TextBkColorActiveRow:=RGB(250, 158, 88);
  Theme.TextBkColorOddRow:=RGB(247, 243, 239);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Green';
  Theme.GradientStartColor:=RGB(243, 253, 188);
  Theme.GradientEndColor:=RGB(124, 180, 34);
  Theme.TitleFontColor:=clBlack;
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=RGB(236, 250, 178);
  Theme.TextBkColorActiveRow:=RGB(201, 248, 48);
  Theme.TextBkColorOddRow:=RGB(230, 249, 151);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Brown';
  Theme.GradientStartColor:=RGB(244, 212, 125);
  Theme.GradientEndColor:=RGB(185, 135, 64);
  Theme.TitleFontColor:=RGB(131, 81, 39);
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=RGB(253, 236, 191);
  Theme.TextBkColorActiveRow:=RGB(236, 204, 119);
  Theme.TextBkColorOddRow:=RGB(230, 249, 151);
  Theme.TextBkColorOddRow:=RGB(252, 230, 170);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Khaki';
  Theme.GradientStartColor:=RGB(151, 136, 111);
  Theme.GradientEndColor:=RGB(114, 99, 77);
  Theme.TitleFontColor:=RGB(217, 217, 187);
  Theme.TextFontColor:=RGB(77, 68, 51);
  Theme.TextBkColor:=RGB(234, 234, 218);
  Theme.TextBkColorActiveRow:=RGB(204, 193, 163);
  GridLookStyleDefs[idx]:=Theme;
end;

procedure LoadGridLookStyleList(L: TStrings);
var
  i: integer;
begin
  L.Clear;
  for i:=0 to High(GridLookStyleDefs) do
    L.Add(GridLookStyleDefs[i].StyleName);
end;

{ return standard color of TabControl according used Theme }
function GetTabBackgroundColorFromBitmap: TColor;
var
  B: TBitmap;
begin
  Result:=clBtnFace;
  if GetThemeEnabled then
  begin
    B:=TBitmap.Create;
    try
      B.Width:=128;
      B.Height:=128;
      GetThemeServices.DrawElement(B.Canvas.Handle, GetThemeServices.GetElementDetails(ttBody), Rect(0, 0, B.Width - 1, B.Height - 1), nil);
      Result:=B.Canvas.Pixels[64, 64];
    finally
      B.Free;
    end;
  end;
end;

{ init all global configuration values }
procedure InitCfgValues;
begin
  RosiCompConfig.Global_CustomGlyphs:=nil;
  RosiCompConfig.Global_PropSaveFile:=nil;
  RosiCompConfig.Global_ProcOnHelp:=nil;
  RosiCompConfig.Global_HTMLHintComponent:=nil;
  RosiCompConfig.Global_HTMLHintPicture:=nil;
  RosiCompConfig.Global_HelpDirectory:=GetAppPath + 'help\';

  RosiCompConfig.rDBToolCfg.SQLScript_StringFieldQuotes:='''';
  RosiCompConfig.rDBToolCfg.SQLScript_CommandSeparator:='GO';
  RosiCompConfig.rDBToolCfg.SQLScript_BooleanTrueValue:='1';
  RosiCompConfig.rDBToolCfg.SQLScript_BooleanFalseValue:='0';
  RosiCompConfig.rDBToolCfg.FileDialog_Options:=[ofEnableSizing, ofNoChangeDir, ofFileMustExist];
  RosiCompConfig.rDBToolCfg.FileDialog_InitDialogEvent:=nil;
  RosiCompConfig.rDBToolCfg.rDlg_DefaultFont:='Tahoma';
  RosiCompConfig.rDBToolCfg.rDlg_DefaultSize:=8;
  RosiCompConfig.rDBToolCfg.rDlg_AnchorClick:=nil;
  RosiCompConfig.rDBToolCfg.rDlg_GridDrawingStyle:=gdsGradient;
  RosiCompConfig.rDBToolCfg.rDlg_BlockStoringGridFields:=false;
  RosiCompConfig.rDBToolCfg.rpictool_LastFileName:='';
  RosiCompConfig.rDBToolCfg.BlobField_ftBlobAsMemo:=true;

  RosiCompConfig.rDBFieldDefaultsCfg.DefaultFloatDisplayFormat:='# ### ##0.00';
  RosiCompConfig.rDBFieldDefaultsCfg.DefaultFloatEditFormat:='0.00';
  RosiCompConfig.rDBFieldDefaultsCfg.DefaultBoolDisplayFormat:='Yes;No';
  RosiCompConfig.rDBFieldDefaultsCfg.DefaultDateTimeDisplayFormat:='';
  RosiCompConfig.rDBFieldDefaultsCfg.DefaultTimeDisplayFormat:='h:nn';
  RosiCompConfig.rDBFieldDefaultsCfg.InternalSQLLog:=nil;
  RosiCompConfig.rDBFieldDefaultsCfg.InternalLogSeparator:=';';

  RosiCompConfig.PropSaveCfg.Suffix_BlockSaveItems:='_nostore';
  RosiCompConfig.PropSaveCfg.FieldList_UseNewWidth:='';
  RosiCompConfig.PropSaveCfg.Grids_UseGlobalSetting:=false;

  RosiCompConfig.rGroupBoxCfg.DefaultCaptionColor:=clGray;
  RosiCompConfig.rGroupBoxCfg.DefaultFontColor:=clWhite;
  RosiCompConfig.rGroupBoxCfg.DefaultCaptionHeight:=14;
  RosiCompConfig.rGroupBoxCfg.DefaultBkColor:=GetTabBackgroundColorFromBitmap;
  RosiCompConfig.rGroupBoxCfg.UseSimpleTabColor:=false;

  RosiCompConfig.rDBComponentsCfg.JPGAsDefault:=rpictool.IsJPGSupported; // and not rpictool.IsPNGSupported;
  RosiCompConfig.rDBComponentsCfg.VisibilityFromField_AutoHide:=false;
  RosiCompConfig.rDBComponentsCfg.VisibilityFromField_AutoShow:=false;

  RosiCompConfig.rDBGridCfg.DefaultDrawingStyle:=gdsThemed; // gdsGradient;
  RosiCompConfig.rDBGridCfg.DefaultGridLookStyle:=grs_Default;
  RosiCompConfig.rDBGridCfg.DefaultGlyphLookStyle:=glsStandard; //glsStandard, glsFlatBlack, glsFlatGray
  RosiCompConfig.rDBGridCfg.FilterListUseColumnAlignment:=true;
  RosiCompConfig.rDBGridCfg.FilterListBoldSelectAll:=true;
  RosiCompConfig.rDBGridCfg.CellSingleLineEOLSeparator:=' | ';

  RosiCompConfig.rDBFieldsCfg.OnSetDefaultGridColumns:=nil;
  RosiCompConfig.rDBFieldsCfg.SetFixedCol:=false;

  RosiCompConfig.rDBFilterCfg.LikeChar:='*';
  RosiCompConfig.rDBFilterCfg.CompoundLikeChar:='%';
  RosiCompConfig.rDBFilterCfg.NullEqualStr:='=';
  RosiCompConfig.rDBFilterCfg.DateFormat:='';
  RosiCompConfig.rDBFilterCfg.BooleanString[true]:='true';
  RosiCompConfig.rDBFilterCfg.BooleanString[false]:='false';
  RosiCompConfig.rDBFilterCfg.TableQuoteStart:='[';
  RosiCompConfig.rDBFilterCfg.TableQuoteStop:=']';
  RosiCompConfig.rDBFilterCfg.FieldQuoteStart:='[';
  RosiCompConfig.rDBFilterCfg.FieldQuoteStop:=']';
  RosiCompConfig.rDBFilterCfg.ShowDelQuotes:=false;
  RosiCompConfig.rDBFilterCfg.ShowCaseSensitive:=true;

  RosiCompConfig.rDBFindCfg.ShowDelQuotes:=false;
  RosiCompConfig.rDBFindCfg.OnFoundEvent:=nil;
  RosiCompConfig.rDBFindCfg.LastOptIndex:=2;
  RosiCompConfig.rDBFindCfg.LastCaseSens:=false;
  RosiCompConfig.rDBFindCfg.LastFindStr:='';

  RosiCompConfig.rDBGridPropsCfg.HideCellParams:=false;
  RosiCompConfig.rDBGridPropsCfg.HideTitleFilterBtn:=false;
  RosiCompConfig.rDBGridPropsCfg.HideGridStyles:=false;

  RosiCompConfig.rDBExportCfg.UseDisableControl:=true;

  RosiCompConfig.rDBPrintCfg.PrintData_Dir:='';
  RosiCompConfig.rDBPrintDirectCfg.HMarginMM:=20;
  RosiCompConfig.rDBPrintDirectCfg.VMarginMM:=20;
  RosiCompConfig.rDBPrintDirectCfg.LineSpaceRatio:=0.2;
  RosiCompConfig.rDBPrintDirectCfg.PreviewMaxWidth:=1600;

  RosiCompConfig.rDBSortCfg.SortFieldList:=true;
  RosiCompConfig.rDBSortCfg.DefaultPath:=GetAppPath;

  RosiCompConfig.RTFRepDlgCfg.LastLangVerIndex:=-1;
  RosiCompConfig.RTFRepDlgCfg.ExportExcelNoFileName:=true;
end;

function TrDBComponentsCfg.DoExitControl(Sender: TControl; F: TField): boolean;
begin
  Result:=true;
  if Assigned(RosiCompConfig.rDBComponentsCfg.ExitControlEvent) and Assigned(F) and F.Required then
  begin
    if F.IsNull or (F.AsString='') then
    begin
      RosiCompConfig.rDBComponentsCfg.ExitControlEvent(Sender, F, Result);
      if not Result then
        THackWinControl(Sender).SetFocus;
    end;
  end;
end;

initialization

InitCfgValues;
InitGridThemeColors;
LoadStandardGlyphs;

finalization

FreeStandardGlyphs;

end.
